/** @file
 * @brief repeating time-points and intervals 
 *
 * @copyright Copyright &copy; 2009 by Intra2net AG
 * @license commercial
 * @contact info@intra2net.com
 *
 */

#ifndef __CRON_HPP
#define __CRON_HPP

#include <time.h>
#include <week.hpp>

namespace I2n {
namespace Time {

/**
    @brief Time points and intervals repeating each week
    This class represents recurring time points and intervals
    which can be repeated on configurable days of the week.
*/
class WeekCron
{
    public:
        static const time_t StNimmerleinsDay;

    private:
        /// Start point of time in seconds since the start of the day
        int Begin;
        /// End point of time in seconds since the start of the day. Only used when #Every != -1
        int End;
        /// Repeat event every xxx seconds in the half-open interval of #Begin and #End. -1 is disabled
        int Every;
        /// Stores the active days this WeekCron is valid for
        I2n::Time::Week Week;

        time_t get_next_point(const time_t calc_from, const int daysec, const bool todaycheck) const;
        time_t get_previousnow_point(const time_t calc_from, const int daysec, const bool todaycheck) const;
        void fill_tm_with_wallclock(struct tm *ft, const int daysec) const;

    public:
        WeekCron();
        WeekCron(const std::string& daystring, const int begin);
        WeekCron(const std::string& daystring, const int begin, const int end, const int every);
        WeekCron(const I2n::Time::Week& week, const int begin);
        WeekCron(const I2n::Time::Week& week, const int begin, const int end, const int every);

        // accessor functions
        I2n::Time::Week get_week(void)
            { return Week; }
        void set_week(const I2n::Time::Week& week)
            { Week=week; }
        int get_begin(void)
            { return Begin; }
        void set_begin(const int begin)
            { Begin=begin; }
        int get_end(void)
            { return End; }
        void set_end(const int end)
            { End=end; }
        int get_every(void)
            { return Every; }
        void set_every(const int every)
            { Every=every; }

        bool is_sane() const;

        time_t get_next_run(time_t calc_from=0) const;
};

}
}

#endif
