/** @file
 * test cases for loggin module.
 *
 * (c) Copyright 2007 by Intra2net AG
 * 
 * info@intra2net.com
 */

//#define NOISEDEBUG

#include <string>
#include <iostream>
#include <iomanip>
#include <exception>

#define BOOST_TEST_DYN_LINK
#include <boost/test/unit_test.hpp>

#include <logfunc.hpp>
#include <tracefunc.hpp>


#ifdef NOISEDEBUG
#define DOUT(msg) std::cout << msg << std::endl
#else
#define DOUT(msg) do {} while (0)
#endif

using namespace I2n;

namespace {

Logger::PartLogger module_logger(HERE);

} // eo namespace <anonymous>

class TestLoggingFixture
{
public:
    TestLoggingFixture()
    {
    }

    ~TestLoggingFixture()
    {
        Logger::set_log_level(0);
        Logger::enable_syslog(false);
        Logger::enable_stderr_log(false);
        Logger::enable_log_file(false);
    }
};

BOOST_FIXTURE_TEST_SUITE(TestLogging, TestLoggingFixture)

BOOST_AUTO_TEST_CASE(Syslog1)
{
    //Logger::enable_syslog("I2N Unittest", Logger::Facility::User );
    Logger::enable_syslog( Logger::Facility::User );
    Logger::PartLogger log(__func__);

    Logger::set_log_level( 7 );
    Logger::enable_stderr_log();
    Logger::enable_log_file("zzUnitTest.log");

    log.error("Test error msg");
    log.error() << "Stream test error msg #" << 2 << ".";
    log.warning() << "Stream test warning msg";
    log.notice() << "Stream test notice msg";
    log.info() << "Stream test info msg";
    log.debug() << "Stream test debug msg";

    log.debug()
        << "multiline log message" << std::endl
        << "this (second) line should be indented" << std::endl
        << "and this also!" << std::endl;
    log.debug(HERE) << "This should have a source info";

    module_logger.debug(HERE) << "module level debug message with source loc info";
} // eo Syslog1()



BOOST_AUTO_TEST_CASE(TestScopeTrace1)
{
    //Logger::enable_syslog("I2N Unittest", Logger::Facility::User );
    Logger::enable_syslog( Logger::Facility::User );
    Logger::PartLogger log(__func__);

    Logger::set_log_level( 7 );
    Logger::enable_stderr_log();
    Logger::enable_log_file("zzUnitTest.log");

    SCOPETRACKER();

    log.notice() << "Stream test notice msg";
    {
        SCOPETRACKER(); // #2
        {
            SCOPETRACKER(); // #3
            {
                SCOPETRACKER(); // #4
            }
        }
        {
            SCOPETRACKER(); // #3
        }
    }
} // eo TestScopeTrace1()

BOOST_AUTO_TEST_SUITE_END()
